@@ -1,6 +1,6 @@
 /* audiocd.c - handle Audio-CD's
-
- *  Copyright (C)2014 J. Lemmens
+ *
+ *  Copyright (C)2015 J. Lemmens
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -57,7 +57,7 @@ void get_cddb_info (misc_t *misc, daisy_t *daisy)
 // some titles are spanned over more lines
          if (strstr (str, l) == NULL)
             continue;
-         strncpy (daisy[i].label, strchr (str, '=') + 1, MAX_PHRASE_LEN - 1);
+         strncpy (daisy[i].label, strchr (str, '=') + 1, 80);
          if (strchr (daisy[i].label, '\n'))
             *strchr (daisy[i].label, '\n') = 0;
          if (strchr (daisy[i].label, '\r'))
@@ -129,10 +129,3 @@ void get_toc_audiocd (misc_t *misc, daisy_t *daisy)
    if (misc->cddb_flag != 'n')
       get_cddb_info (misc, daisy);
 } // get_toc_audiocd
-
-void set_drive_speed (misc_t *misc, int drive_speed)
-{
-   if (misc->cd_type == CDIO_DISC_MODE_CD_DA || \
-       misc->cd_type == CDIO_DISC_MODE_CD_DATA)
-      cdio_set_speed (misc->p_cdio, drive_speed);
-} // set_drive_speed
