@@ -1,5 +1,5 @@
 /* header file for daisy-player
- *  Copyright (C)2014 J. Lemmens
+ *  Copyright (C)2015 J. Lemmens
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -38,25 +38,36 @@
 #include <errno.h>
 #include <time.h>
 #include <sys/ioctl.h>
-#include <libxml2/libxml/xmlreader.h>
-#include <libxml2/libxml/xmlwriter.h>
+#include <libxml/xmlreader.h>
+#include <libxml/xmlwriter.h>
+#include <libxml/HTMLparser.h>
 #include <cdio/cdio.h>
 #include <cdio/cdda.h>
 #include <cdio/paranoia.h>
 #include <cdio/disc.h>
 #include <magic.h>
 
-#define MAX_PHRASE_LEN 2000
+#undef PACKAGE
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_URL
+#undef PACKAGE_VERSION
+#undef VERSION
+#include "config.h"
+
+// jos #define MAX_PHRASE_LEN 2000
 #define MAX_CMD 512
 #define MAX_STR 256
 #define MAX_TAG 1024
 
 typedef struct Daisy
 {
-   int playorder, x, y, screen, n_phrases;
+   int playorder, x, y, screen;
    float begin, duration;
    char smil_file[MAX_STR], anchor[MAX_STR], class[MAX_STR];
-   char label[MAX_PHRASE_LEN];
+   char label[100];
    int level, page_number;
    char daisy_mp[MAX_STR]; // discinfo
    char filename[MAX_STR]; // Audio-CD
@@ -96,25 +107,30 @@ typedef struct Misc
    int current, max_y, max_x, total_items, level, displaying, ignore_bookmark;
    int phrase_nr, tts_no, depth, total_phrases, total_pages;
    int pipefd[2], tmp_wav_fd;
+   int pause_resume_playing;
+   float pause_resume_clip_begin;
    float speed, total_time;
-   float clip_begin, clip_end, start_time;
+   float clip_begin, clip_end;
    xmlTextReaderPtr reader;
    pid_t daisy_player_pid, player_pid;
    char NCC_HTML[MAX_STR], ncc_totalTime[MAX_STR];
    char daisy_version[MAX_STR], daisy_title[MAX_STR], daisy_language[MAX_STR];
    char daisy_mp[MAX_STR];
-   char tag[MAX_TAG], label[MAX_PHRASE_LEN];
-   char bookmark_title[MAX_STR], search_str[MAX_STR];
+   char tag[MAX_TAG];
+   char *label;
+   int label_len;
+   char bookmark_title[MAX_STR];
+   char search_str[MAX_STR];
    char *wd, cd_dev[MAX_STR], sound_dev[MAX_STR];
    char cddb_flag, opf_name[MAX_STR], ncx_name[MAX_STR];
    char current_audio_file[MAX_STR], tmp_wav[MAX_STR + 1], mcn[MAX_STR];
-   time_t seconds;
+   time_t elapsed_seconds;
    WINDOW *screenwin, *titlewin;
    cdrom_paranoia_t *par;
    cdrom_drive_t *drv;
    CdIo_t *p_cdio;
    int cd_type;
-   lsn_t lsn_cursor;
+   lsn_t lsn_cursor, pause_resume_lsn_cursor;
 } misc_t;
 
 extern void save_rc (misc_t *);
@@ -144,3 +160,4 @@ extern void view_screen (misc_t *, daisy_t *);
 extern daisy_t *create_daisy_struct (misc_t *, my_attribute_t *);
 extern daisy_t *get_number_of_tracks (misc_t *);
 extern void failure (char *, int);
+extern void put_bookmark (misc_t *);
