@@ -1,5 +1,5 @@
 /* daisy3.c - functions to insert daisy3 info into a struct.
- *  Copyright (C)2014 J. Lemmens
+ *  Copyright (C)2015 J. Lemmens
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -22,7 +22,7 @@ void failure (char *str, int e)
 {
    endwin ();
    beep ();
-   printf ("%s: %s\n", str, strerror (e));
+   printf ("\n%s: %s\n\n", str, strerror (e));
    fflush (stdout);
    _exit (-1);
 } // failure
@@ -112,6 +112,8 @@ void get_clips (misc_t *misc, char *orig_begin, char *end)
 
    if (misc->cd_type == CDIO_DISC_MODE_CD_DA)
       return;
+   if (! *orig_begin)
+      return;
    strncpy (begin_str, orig_begin,  MAX_STR - 1);
    begin = begin_str;
    while (! isdigit (*begin))
@@ -120,7 +122,7 @@ void get_clips (misc_t *misc, char *orig_begin, char *end)
       *strchr (begin, 's') = 0;
    if (! strchr (begin, ':'))
       misc->clip_begin = (float) atof (begin);
-   else
+   else         
       misc->clip_begin = read_time (begin);
 
 // fill end
@@ -140,65 +142,63 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
    char attr[MAX_STR];
 
    snprintf (attr, MAX_STR - 1, "%s", (char *)
-        xmlTextReaderGetAttribute (reader, (const xmlChar *) "class"));
+        xmlTextReaderGetAttribute (reader, BAD_CAST "class"));
    if (strcmp (attr, "(null)"))
       snprintf (my_attribute->class, MAX_STR - 1, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*) xmlTextReaderGetAttribute
-                          (reader, (const xmlChar *) "clip-begin"));
-   if (strcmp (attr, "(null)"))
-      snprintf (my_attribute->clip_begin, MAX_STR - 1,
-                "%s", attr);
-   snprintf (attr, MAX_STR - 1, "%s", (char*)
-             xmlTextReaderGetAttribute (reader,
-                    (const xmlChar *) "clipBegin"));
+                          (reader, BAD_CAST "clip-begin"));
    if (strcmp (attr, "(null)"))
       snprintf (my_attribute->clip_begin, MAX_STR - 1, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "clip-end"));
+           xmlTextReaderGetAttribute (reader, BAD_CAST "clipBegin"));
+   if (strcmp (attr, "(null)"))
+      snprintf (my_attribute->clip_begin, MAX_STR - 1, "%s", attr);
+   snprintf (attr, MAX_STR - 1, "%s", (char*)
+       xmlTextReaderGetAttribute (reader, BAD_CAST "clip-end"));
    if (strcmp (attr, "(null)"))
       snprintf (my_attribute->clip_end, MAX_STR - 1, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "clipEnd"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "clipEnd"));
    if (strcmp (attr, "(null)"))
       snprintf (my_attribute->clip_end, MAX_STR - 1, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "href"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "href"));
    if (strcmp (attr, "(null)"))
       snprintf (my_attribute->href, MAX_STR - 1, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "id"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "id"));
    if (strcmp (attr, "(null)"))
       snprintf (my_attribute->id, MAX_STR - 1, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "idref"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "idref"));
    if (strcmp (attr, "(null)"))
       snprintf (my_attribute->idref, MAX_STR - 1, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "item"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "item"));
    if (strcmp (attr, "(null)"))
       misc->current = atoi (attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "level"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "level"));
    if (strcmp (attr, "(null)"))
       misc->level = atoi ((char *) attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "media-type"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "media-type"));
    if (strcmp (attr, "(null)"))
       snprintf (my_attribute->media_type, MAX_STR - 1, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "name"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "name"));
    if (strcmp (attr, "(null)"))
    {
       char name[MAX_STR], content[MAX_STR];
 
       *name = 0;
       snprintf (attr, MAX_STR - 1, "%s", (char *)
-                xmlTextReaderGetAttribute (reader, (const xmlChar *) "name"));
+                xmlTextReaderGetAttribute (reader, BAD_CAST "name"));
       if (strcmp (attr, "(null)"))
          snprintf (name, MAX_STR - 1, "%s", attr);
       *content = 0;
       snprintf (attr, MAX_STR - 1, "%s", (char *)
-                xmlTextReaderGetAttribute (reader, (const xmlChar *) "content"));
+                xmlTextReaderGetAttribute (reader, BAD_CAST "content"));
       if (strcmp (attr, "(null)"))
          snprintf (content, MAX_STR - 1, "%s", attr);
       if (strcasestr (name, "dc:format"))
@@ -233,68 +233,71 @@ void get_attributes (misc_t *misc, my_attribute_t *my_attribute,
       } // if
    } // if
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "playorder"));
-   if (strcmp (attr, "(null)"))                                  
+       xmlTextReaderGetAttribute (reader, BAD_CAST "playorder"));
+   if (strcmp (attr, "(null)"))
       snprintf (my_attribute->playorder, MAX_STR - 1, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "phrase"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "phrase"));
    if (strcmp (attr, "(null)"))
       misc->phrase_nr = atoi ((char *) attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "seconds"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "seconds"));
    if (strcmp (attr, "(null)"))
    {
-      misc->seconds = atoi (attr);
-      if (misc->seconds < 0)
-         misc->seconds = 0;
+      misc->elapsed_seconds = atoi (attr);
+      if (misc->elapsed_seconds < 0)
+         misc->elapsed_seconds = 0;
+      if (misc->elapsed_seconds >= misc->total_time)
+         misc->elapsed_seconds = 0;
    } // if
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-          xmlTextReaderGetAttribute (reader, (const xmlChar *) "smilref"));
+          xmlTextReaderGetAttribute (reader, BAD_CAST "smilref"));
    if (strcmp (attr, "(null)"))
       snprintf (my_attribute->smilref, MAX_STR - 1, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "sound_dev"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "sound_dev"));
    if (strcmp (attr, "(null)"))
       snprintf (misc->sound_dev, MAX_STR, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "cd_dev"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "cd_dev"));
    if (strcmp (attr, "(null)"))
       snprintf (misc->cd_dev, MAX_STR, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "cddb_flag"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "cddb_flag"));
    if (strcmp (attr, "(null)"))
       misc->cddb_flag = (char) attr[0];
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "speed"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "speed"));
    if (strcmp (attr, "(null)"))
       misc->speed = atof (attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "src"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "src"));
    if (strcmp (attr, "(null)"))
       snprintf (my_attribute->src, MAX_STR - 1, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "tts"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "tts"));
    if (strcmp (attr, "(null)"))
       misc->tts_no = atof ((char *) attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "toc"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "toc"));
    if (strcmp (attr, "(null)"))
       snprintf (my_attribute->toc, MAX_STR - 1, "%s", attr);
    snprintf (attr, MAX_STR - 1, "%s", (char*)
-       xmlTextReaderGetAttribute (reader, (const xmlChar *) "value"));
+       xmlTextReaderGetAttribute (reader, BAD_CAST "value"));
    if (strcmp (attr, "(null)"))
       snprintf (my_attribute->value, MAX_STR - 1, "%s", attr);
-} // get_attributes
+} // get_attributes                               
 
 int get_tag_or_label (misc_t *misc, my_attribute_t *my_attribute,
                       xmlTextReaderPtr reader)
 {
    int type;
 
-   *misc->tag =  *misc->label = 0;
-   *my_attribute->class = *my_attribute->clip_begin = *my_attribute->clip_end =
-   *my_attribute->href = *my_attribute->id = *my_attribute->media_type =
-   *my_attribute->playorder = * my_attribute->smilref = *my_attribute->src =
+   *misc->tag = 0;
+   *my_attribute->class = *my_attribute->clip_begin =
+   *my_attribute->clip_end = *my_attribute->href = *my_attribute->id =
+   *my_attribute->media_type = *my_attribute->playorder =
+   * my_attribute->smilref = *my_attribute->src =
    *my_attribute->toc = *my_attribute->value = 0;
 
    if (reader == NULL)
@@ -317,7 +320,9 @@ int get_tag_or_label (misc_t *misc, my_attribute_t *my_attribute,
    case -1:
       endwin ();
       beep ();
-      printf (gettext ("\nCannot read type: %d\n"), type);
+      printf ("\n");
+      printf (gettext ("Cannot read type: %d"), type);
+      printf ("\n");
       fflush (stdout);
       _exit (1);
    case XML_READER_TYPE_ELEMENT:
@@ -343,9 +348,13 @@ int get_tag_or_label (misc_t *misc, my_attribute_t *my_attribute,
          else
             break;
       } // while
+      misc->label_len = 
+                  strlen ((char *) xmlTextReaderConstValue (reader)) + x + 10;
+      misc->label = malloc (misc->label_len);
       strncpy (misc->label, (char *) xmlTextReaderConstValue (reader) + x,
-                      MAX_PHRASE_LEN);
-      for (x = strlen (misc->label) - 1; x >= 0 && isspace (misc->label[x]); x--)
+               misc->label_len - 1);
+      for (x = strlen (misc->label) - 1;
+           x >= 0 && isspace (misc->label[x]); x--)
          misc->label[x] = 0;
       for (x = 0; misc->label[x] > 0; x++)
          if (! isascii (misc->label[x]))
@@ -382,7 +391,9 @@ void parse_text_file (misc_t *misc, my_attribute_t *my_attribute,
    {
       endwin ();
       beep ();
-      printf (gettext ("\nCannot read %s\n"), text_file);
+      printf ("\n");
+      printf (gettext ("Cannot read %s"), text_file);
+      printf ("\n");
       fflush (stdout);
       _exit (1);
    } // if
@@ -433,7 +444,9 @@ void get_page_number_3 (misc_t *misc, my_attribute_t *my_attribute)
    {
       endwin ();
       beep ();
-      printf (gettext ("\nCannot read %s\n"), my_attribute->src);
+      printf ("\n");
+      printf (gettext ("Cannot read %s"), my_attribute->src);
+      printf ("\n");
       fflush (stdout);
       _exit (1);
    } // if
@@ -477,7 +490,9 @@ void parse_smil_3 (misc_t *misc, my_attribute_t *my_attribute,
       {
          endwin ();
          beep ();
-         printf (gettext ("\nCannot read %s\n"), daisy[x].smil_file);
+         printf ("\n");
+         printf (gettext ("Cannot read %s"), daisy[x].smil_file);
+         printf ("\n");
          fflush (stdout);
          _exit (1);
       } // if
@@ -498,7 +513,8 @@ void parse_smil_3 (misc_t *misc, my_attribute_t *my_attribute,
 // get misc->clip_begin
          if (strcasecmp (misc->tag, "audio") == 0)
          {
-            get_clips (misc, my_attribute->clip_begin, my_attribute->clip_end);
+            get_clips (misc, my_attribute->clip_begin,
+                       my_attribute->clip_end);
             daisy[x].begin = misc->clip_begin;
             daisy[x].duration += misc->clip_end - misc->clip_begin;
 // get clip_end
@@ -545,7 +561,9 @@ void parse_content (misc_t *misc, my_attribute_t *my_attribute,
    {
       endwin ();
       beep ();
-      printf (gettext ("\nCannot read %s\n"), daisy[misc->current].smil_file);
+      printf ("\n");
+      printf (gettext ("Cannot read %s"), daisy[misc->current].smil_file);
+      printf ("\n");
       fflush (stdout);
       _exit (1);
    } // if
@@ -587,7 +605,9 @@ void parse_xml (misc_t *misc, my_attribute_t *my_attribute,
    {
       endwin ();
       beep ();
-      printf (gettext ("\nCannot read %s\n"), misc->ncx_name);
+      printf ("\n");
+      printf (gettext ("Cannot read %s"), misc->ncx_name);
+      printf ("\n");
       fflush (stdout);
       _exit (1);
    } // if
@@ -606,7 +626,9 @@ void parse_xml (misc_t *misc, my_attribute_t *my_attribute,
    {
       endwin ();
       beep ();
-      printf (gettext ("\nCannot read %s\n"), xml_name);
+      printf ("\n");
+      printf (gettext ("Cannot read %s"), xml_name);
+      printf ("\n");
       fflush (stdout);
       _exit (1);
    } // if
@@ -681,7 +703,9 @@ void parse_manifest (misc_t *misc, my_attribute_t *my_attribute,
    {
       endwin ();
       beep ();
-      printf (gettext ("\nCannot read %s\n"), name);
+      printf ("\n");
+      printf (gettext ("Cannot read %s"), name);
+      printf ("\n");
       fflush (stdout);
       _exit (1);
    } // if
@@ -730,8 +754,7 @@ void parse_manifest (misc_t *misc, my_attribute_t *my_attribute,
       {
          strncpy (daisy[misc->current].smil_file, my_attribute->href,
                   MAX_STR - 1);
-         snprintf (daisy[misc->current].label,
-                   MAX_STR - 1, "%d", misc->current + 1);
+         snprintf (daisy[misc->current].label, 80, "%d", misc->current + 1);
          daisy[misc->current].screen = misc->current / misc->max_y;
          daisy[misc->current].y =
                 misc->current - (daisy[misc->current].screen * misc->max_y);
@@ -758,7 +781,9 @@ void read_daisy_3 (misc_t *misc, my_attribute_t *my_attribute,
    if (*path == 0)
    {
       endwin ();
-      printf (gettext ("\nNo DAISY-CD or Audio-cd found\n"));
+      printf ("\n");
+      printf (gettext ("No DAISY-CD or Audio-cd found"));
+      printf ("\n");
       beep ();
       fflush (stdout);
       usage (PACKAGE);
@@ -782,7 +807,7 @@ void read_daisy_3 (misc_t *misc, my_attribute_t *my_attribute,
 
 void get_label (misc_t *misc, daisy_t *daisy, int indent)
 {
-   strncpy (daisy[misc->current].label, misc->label, MAX_STR - 1);
+   strncpy (daisy[misc->current].label, misc->label, 80);
    daisy[misc->current].label[64 - daisy[misc->current].x] = 0;
    if (misc->displaying == misc->max_y)
       misc->displaying = 1;
@@ -808,7 +833,9 @@ void parse_ncx (misc_t *misc, my_attribute_t *my_attribute,
    {
       endwin ();
       beep ();
-      printf (gettext ("\nCannot read %s\n"), name);
+      printf ("\n");
+      printf (gettext ("Cannot read %s"), name);
+      printf ("\n");
       fflush (stdout);
       _exit (1);
    } // if
@@ -885,8 +912,7 @@ void parse_ncx (misc_t *misc, my_attribute_t *my_attribute,
       if (strcasecmp (misc->tag, "page") == 0 || found_page)
       {
          found_page = 0;
-         strncpy (daisy[++misc->current].label, my_attribute->number,
-                  MAX_PHRASE_LEN - 1);
+         strncpy (daisy[++misc->current].label, my_attribute->number, 80);
          strncpy (daisy[misc->current].smil_file, name, MAX_STR - 1);
          daisy[misc->current].screen = misc->current / misc->max_y;
          daisy[misc->current].y =
@@ -897,7 +923,4 @@ void parse_ncx (misc_t *misc, my_attribute_t *my_attribute,
    xmlTextReaderClose (ncx);
    xmlFreeDoc (doc);
    misc->total_items = misc->current;
-#ifdef EBOOK_SPEAKER
-   check_phrases ();
-#endif
 } // parse_ncx
